//
//  stb_image.c
//  iOSHintView
//
//  Created by Martin Ruckert on 11.10.22.
//

#define STBI_ONLY_JPEG
#define STBI_ONLY_PNG
#define STBI_ONLY_BMP
#define STBI_NO_STDIO
#define STB_IMAGE_IMPLEMENTATION
#include "stb_image.h"
